@@ -8,15 +8,15 @@
 
 G_BEGIN_DECLS
 
-#include <libempathy/empathy-contact.h>
-#define EMPATHY_TYPE_SUBSCRIPTION empathy_subscription_get_type()
-GType empathy_subscription_get_type (void);
 #include <libempathy/empathy-utils.h>
 #define EMPATHY_TYPE_REG_EX_TYPE empathy_reg_ex_type_get_type()
 GType empathy_reg_ex_type_get_type (void);
 #include <libempathy/empathy-message.h>
 #define EMPATHY_TYPE_MESSAGE_TYPE empathy_message_type_get_type()
 GType empathy_message_type_get_type (void);
+#include <libempathy/empathy-contact.h>
+#define EMPATHY_TYPE_CAPABILITIES empathy_capabilities_get_type()
+GType empathy_capabilities_get_type (void);
 G_END_DECLS
 
 #endif /* __LIBEMPATHY_ENUM_TYPES_H__ */
